﻿using DataSharing.RawData;
using DataSharing.SendTask;
using DataSharing.Share.Enums;
using Hotline.Share.Dtos.Order;
using Hotline.Share.Enums.Order;
using MapsterMapper;
using Microsoft.Extensions.Logging;
using System.Security.Cryptography;
using System.Text;
using System.Web;
using XF.Domain.Dependency;
using XF.Domain.Repository;

namespace DataSharing.CityStateDataExchange
{
    /// <summary>
    /// 市州工单互转
    /// </summary>
    public class CityStateDataExchangeService : ICityStateDataExchangeService, IScopeDependency
    {
        private readonly IMapper _mapper;
        private readonly ILogger<CityStateDataExchangeService> _logger;
        private readonly IInitPushDataService _initPushDataService;
        private readonly IRepository<DsUserTokenInfo> _dsUserTokenInfoRepository;
        private readonly ISharingConfigurationManager _sharingConfigurationManager;

        /// <summary>
        /// 
        /// </summary>
        /// <param name="mapper"></param>
        /// <param name="logger"></param>
        /// <param name="initPushDataService"></param>
        /// <param name="dsUserTokenInfoRepository"></param>
        /// <param name="sharingConfigurationManager"></param>
        public CityStateDataExchangeService(IMapper mapper,
            ILogger<CityStateDataExchangeService> logger,
            IInitPushDataService initPushDataService,
            IRepository<DsUserTokenInfo> dsUserTokenInfoRepository,
            ISharingConfigurationManager sharingConfigurationManager)
        {
            _mapper = mapper;
            _logger = logger;
            _initPushDataService = initPushDataService;
            _dsUserTokenInfoRepository = dsUserTokenInfoRepository;
            _sharingConfigurationManager = sharingConfigurationManager;
        }

        /// <summary>
        /// 市州工单互转
        /// </summary>
        /// <param name="dto"></param>
        /// <param name="platformSource"></param>
        /// <param name="cancellationToken"></param>
        /// <returns></returns>
        public async Task InitCityDataAsync(OrderDto dto, string platformSource, CancellationToken cancellationToken)
        {
            var userInfo = await _dsUserTokenInfoRepository.GetAsync(p => p.PlatformSource == platformSource && p.State == "1", cancellationToken);
            if (userInfo is null)
                return;

            string source = userInfo.PlatformSourceName;
            string secretKey = GetMD5(source + userInfo.AccountSecret);

            var serverName = dto.IdentityType == EIdentityType.Enterprise ? "企业" : "市民";
            Dictionary<string, string> dicParam = new()
            {
                { "Code", secretKey },
                { "FSHCode", dto.No },
                { "ServerName", HttpUtility.UrlEncode(serverName)},
                { "PurTypeName",HttpUtility.UrlEncode(dto.AcceptType) },
                { "ConTypeName",HttpUtility.UrlEncode( dto.HotspotSpliceName)},
                { "Title", HttpUtility.UrlEncode(dto.Title)},
                { "LinkName", HttpUtility.UrlEncode(dto.FromName)},
                { "Gender",dto.FromGender switch
                    {
                        EGender.Female => "2",
                        EGender.Male => "1",
                        _ => "0",
                    }
                },
                { "Mobile", dto.Contact},
                { "Mail","" },
                { "Address", HttpUtility.UrlEncode( dto.FullAddress )},
                { "Content", HttpUtility.UrlEncode(dto.Content)},
                { "Source",HttpUtility.UrlEncode( source )}
            };

            var config = _sharingConfigurationManager.GetYiBinConfig().CityHandOver;
            if (config == null)
                return;

            string path = "";
            if (config.IsNew)
                path = config.NewPushOrder;
            else
                path = config.PushOrder;
            //写入待推送数据
            await _initPushDataService.InitPushDataAsync(platformSource, path, System.Text.Json.JsonSerializer.Serialize(dicParam), dto.No, platformSource: EPlatformSource.CityDataExchange, cancellationToken: cancellationToken);

        }

        /// <summary>
        /// 办理意见推送
        /// </summary>
        /// <param name="dto"></param>
        /// <param name="cancellationToken"></param>
        /// <returns></returns>
        public async Task UndertakAdviceAsync(OrderDto dto, CancellationToken cancellationToken)
        {
            string areaCode = _sharingConfigurationManager.GetCityCode();
            //判断推送市州
            string platformSource = "";
            switch (areaCode)
            {
                case "510500":  //转泸州12345
                    platformSource = "CityDataExchangeLz";
                    break;
                case "510300"://转自贡12345
                    platformSource = "CityDataExchangeZG";
                    break;
                case "511500"://转宜宾12345
                    platformSource = "CityDataExchangeYB";
                    break;
                case "511000"://转内江12345
                    platformSource = "CityDataExchangeNJ";
                    break;

                default:
                    break;
            }
            var userInfoSend = await _dsUserTokenInfoRepository.GetAsync(p => p.PlatformSource == platformSource && p.State == "1", cancellationToken);
            if (userInfoSend is null)
                return;
            string source = userInfoSend.PlatformSourceName;
            string secretKey = GetMD5(source + userInfoSend.AccountSecret);

            var userInfo = await _dsUserTokenInfoRepository.GetAsync(p => p.PlatformSource == dto.Source.ToString() && p.State == "1", cancellationToken);
            if (userInfo is null)
                return;

            Dictionary<string, object> dic = new()
            {
                { "Code", secretKey }, //请求Code
                { "SCDRCode", dto.ExternalId }, //工单编号
                { "UndertakAdvice", HttpUtility.UrlEncode(dto.ActualOpinion.Trim()) }, //承办意见
                { "Source", HttpUtility.UrlEncode(source) } //来源
            };

            var config = _sharingConfigurationManager.GetYiBinConfig().CityHandOver;
            if (config == null)
                return;

            string path = "";
            if (config.IsNew)
                path = config.NewPushOpinion;
            else
                path = config.PushOpinion;

            //写入待推送数据
            await _initPushDataService.InitPushDataAsync(userInfo.PlatformSource, path, System.Text.Json.JsonSerializer.Serialize(dic), dto.No, platformSource: EPlatformSource.CityDataExchange, cancellationToken: cancellationToken);

        }

        #region MD5 32位加密
        /// <summary>
        /// MD5 32位加密
        /// </summary>
        /// <param name="myString"></param>
        /// <returns></returns>
        private static string GetMD5(string myString)
        {
            string cl = myString;
            string pwd = "";
            try
            {
                //实例化一个md5对像
                MD5 md5 = MD5.Create();
                // 加密后是一个字节类型的数组，这里要注意编码UTF8/Unicode等的选择　
                byte[] s = md5.ComputeHash(Encoding.UTF8.GetBytes(cl));
                // 通过使用循环，将字节类型的数组转换为字符串，此字符串是常规字符格式化所得
                for (int i = 0; i < s.Length; i++)
                {
                    // 将得到的字符串使用十六进制类型格式。格式后的字符是小写的字母，如果使用大写（X）则格式后的字符是大写字符 
                    pwd = pwd + s[i].ToString("X2");
                }
            }
            catch (Exception)
            {
            }
            return pwd;
        }

        #endregion
    }
}
